/*-
 * Copyright (c) 2013 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * structure taken from OpenSolaris fcntl.h
 */

#ifndef _LONGPATH_H
#define _LONGPATH_H

#include "longpath_config.h"

/*
 * XXX HACK: This prevents the solaris includes to win the redefine_extname race
 *	     -- is there a better way ?
 */
#if defined(_LONGPATH_REDEFINE_PRE_H) && defined(__PRAGMA_REDEFINE_EXTNAME)
#define __PRAGMA_REDEFINE_EXTNAME_SAVE	
#undef  __PRAGMA_REDEFINE_EXTNAME
#endif

// XXX conditional includes incomplete
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
// mktemp
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <utime.h>	// utimbuf

// stdlib stuff
#include <dirent.h>

#ifdef  __PRAGMA_REDEFINE_EXTNAME_SAVE
#undef  __PRAGMA_REDEFINE_EXTNAME_SAVE
#define __PRAGMA_REDEFINE_EXTNAME
#endif

#ifdef  __cplusplus
extern "C" {
#endif

#ifdef DEBUG_LF

#ifdef _FILE_OFFSET_BITS
# if (_FILE_OFFSET_BITS == 64)
#  warning _FILE_OFFSET_BITS == 64
# elif (_FILE_OFFSET_BITS == 32)
#  warning _FILE_OFFSET_BITS == 32
# else
#  warning _FILE_OFFSET_BITS other value
# endif
#else 
#warning _FILE_OFFSET_BITS UN_defined
#endif

#ifdef _LARGEFILE64_SOURCE
#warning _LARGEFILE64_SOURCE defined
#else
#warning _LARGEFILE64_SOURCE UN_defined
#endif

#ifdef _LARGEFILE_SOURCE
#warning _LARGEFILE_SOURCE defined
#else
#warning _LARGEFILE_SOURCE UN_defined
#endif

#ifdef _LP64
#warning _LP64 defined
#else
#warning _LP64 UN_defined
#endif

#endif /* DEBUG_LF */

/* from solaris sys/feature_tests.h */
#if defined(_LP64)
#ifndef _FILE_OFFSET_BITS
#define _FILE_OFFSET_BITS       64
#endif
#if     _FILE_OFFSET_BITS - 0 != 64
#error  "invalid _FILE_OFFSET_BITS value specified"
#endif
#else   /* _LP64 */
#ifndef _FILE_OFFSET_BITS
#define _FILE_OFFSET_BITS       32
#endif
#if     _FILE_OFFSET_BITS - 0 != 32 && _FILE_OFFSET_BITS - 0 != 64
#error  "invalid _FILE_OFFSET_BITS value specified"
#endif
#endif  /* _LP64 */


/* large file compilation environment setup */

/* func -> func64 */
#if !defined(_LP64) && _FILE_OFFSET_BITS == 64
 #ifdef __PRAGMA_REDEFINE_EXTNAME
  #pragma redefine_extname        openl		openl64
  #pragma redefine_extname        creatl	creatl64
  #pragma redefine_extname        statl		statl64
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #pragma redefine_extname        openatl	openatl64
#ifdef HAVE_ATTROPEN
   #pragma redefine_extname        attropenl	attropenl64
#endif
   #pragma redefine_extname        fstatatl	fstatatl64
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #pragma redefine_extname        lstatl	lstatl64
  #endif
  #pragma redefine_extname        mkstempl	mkstempl64
  #pragma redefine_extname        mkstempsl	mkstempsl64
  #if defined(_GNU_SOURCE)
   #pragma redefine_extname        mkostempl	mkostempl64
   #pragma redefine_extname        mkostempsl	mkostempsl64
  #endif
 #else /* __PRAGMA_REDEFINE_EXTNAME */
  #define        openl		openl64
  #define        creatl		creatl64
  #define        statl		statl64
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #define        openatl	openatl64
#ifdef HAVE_ATTROPEN
   #define        attropenl	attropenl64
#endif
   #define        fstatatl	fstatatl64
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #define        lstatl	lstatl64
  #endif
  #define        mkstempl	mkstempl64
  #define        mkstempsl	mkstempsl64
  #if defined(_GNU_SOURCE)
   #define        mkostempl	mkostempl64
   #define        mkostempsl	mkostempsl64
  #endif
 #endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* !_LP64 && _FILE_OFFSET_BITS == 64 */

/* func64 -> func */
#if defined(_LP64) && defined(_LARGEFILE64_SOURCE)
 #ifdef __PRAGMA_REDEFINE_EXTNAME
  #pragma redefine_extname        openl64	openl
  #pragma redefine_extname        creatl64	creatl
  #pragma redefine_extname        statl64	statl
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #pragma redefine_extname        openatl64	openatl
#ifdef HAVE_ATTROPEN
   #pragma redefine_extname        attropenl64	attropenl
#endif
   #pragma redefine_extname        fstatatl64	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #pragma redefine_extname        lstatl64	lstatl
  #endif
  #pragma redefine_extname        mkstempl64	mkstempl
  #pragma redefine_extname        mkstempsl64	mkstempsl
  #if defined(_GNU_SOURCE)
   #pragma redefine_extname        mkostempl64	mkostempl
   #pragma redefine_extname        mkostempsl64	mkostempsl
  #endif
 #else /* __PRAGMA_REDEFINE_EXTNAME */
  #define        openl64	openl
  #define        creatl64	creatl
  #define        statl64	statl
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #define        openatl64	openatl
#ifdef HAVE_ATTROPEN
   #define        attropenl64	attropenl
#endif
   #define        fstatatl64	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #define        lstatl64	lstatl
  #endif
  #define        mkstempl64	mkstempl
  #define        mkstempsl64	mkstempsl
  #if defined(_GNU_SOURCE)
   #define        mkostempl64	mkostempl
   #define        mkostempsl64	mkostempsl
  #endif
 #endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* _LP64 && _LARGEFILE64_SOURCE */

#if defined(__STDC__)
extern int openl(const char *, int, ...);
extern int creatl(const char *, mode_t);
extern int statl(const char *, struct stat *);
extern int linkl(const char *, const char *);
extern int unlinkl(const char *);
extern int chownl(const char *, uid_t , gid_t);
extern int chmodl(const char *, mode_t);
extern int mknodl(const char *, mode_t , dev_t);
extern int mkfifol(const char *, mode_t);
extern int rmdirl(const char *);
extern int mkdirl(const char *,  mode_t);
extern int renamel(const char *, const char *);
extern int utimesl(const char *, const struct timeval *);
extern int lutimesl(const char *, const struct timeval *);
extern int utimel(const char *, const struct utimbuf *);

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
extern int openatl(int , const char *, int, ...);
#ifdef HAVE_ATTROPEN
extern int attropenl(const char *, const char *, int , ...);
#endif
extern int fstatatl(int, const char *, struct stat *, int);
#if defined(HAVE_LINKAT)
extern int linkatl(int , const char *, int , const char *, int);
#endif
extern int unlinkatl(int , const char *, int);
#if defined(HAVE_SYMLINKAT)
extern int symlinkatl(const char *, int , const char *);
#endif
#if defined(HAVE_READLINKAT)
extern int readlinkatl(int , const char *, char *, size_t);
#endif
extern int fchownatl(int , const char *, uid_t , gid_t , int);
#if defined(HAVE_MKNODAT)
extern int mknodatl(int , const char *, mode_t , dev_t);
#endif
#if defined(HAVE_MKFIFOAT)
extern int mkfifoatl(int , const char *, mode_t);
#endif
extern int rmdiratl(int , const char *);
#if defined(HAVE_MKDIRAT)
extern int mkdiratl(int , const char *, mode_t);
#endif
extern int renameatl(int , const char *, int , const char *);
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
extern int futimesatl(int , const char *, const struct timeval *);
extern int utimensatl(int, const char *, const struct timespec *, int);
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
extern int lstatl(const char *, struct stat *);
extern int symlinkl(const char *, const char *);
extern int readlinkl(const char *, char *, size_t);
extern int lchownl(const char *, uid_t , gid_t);
#endif

/* mkstemp */
extern char *mktempl(char *);
extern char *mkdtempl(char *);
extern int mkstempl(char *);
extern int mkstempsl(char *, int);
/* mkstemp GNU */
#if defined(_GNU_SOURCE)
extern int mkostempl(char *, int);
extern int mkostempsl(char *, int , int);
#endif

DIR *opendirl(const char *);

/* linux */
#ifdef HAVE_FGETXATTR
extern ssize_t getxattrl(const char *, const char *, void *, size_t);
extern ssize_t lgetxattrl(const char *, const char *, void *, size_t);
extern int setxattrl(const char *, const char *, const void *, size_t, int);
extern int lsetxattrl(const char *, const char *, const void *, size_t, int);
#endif

/* 64 */
#if     defined(_LARGEFILE64_SOURCE) && !((_FILE_OFFSET_BITS == 64) && \
            !defined(__PRAGMA_REDEFINE_EXTNAME))
extern int openl64(const char *, int , ...);
extern int creatl64(const char *, mode_t);
extern int statl64(const char *, struct stat64 *);

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
extern int openatl64(int , const char *, int, ...);
#ifdef HAVE_ATTROPEN
extern int attropenl64(const char *, const char *, int , ...);
#endif
extern int fstatatl64(int , const char *, struct stat64 *, int);
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
extern int lstatl64(const char *, struct stat64 *);
#endif

/* 64 mkstemp*/
extern int mkstempl64(char *);
extern int mkstempsl64(char *, int);
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
extern int mkostempl64(char *, int);
extern int mkostempsl64(char *, int , int);
#endif
#endif /* 64 */
#else	/* defined(__STDC__) */

extern int openl();
extern int creatl();
extern int statl();
extern int linkl();
extern int unlinkl();
extern int chownl();
extern int chmodl();
extern int mknodl();
extern int mkfifol();
extern int rmdirl();
extern int mkdirl();
extern int renamel();
extern int utimesl();
extern int lutimesl();
extern int utimel();

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
extern int openatl();
#ifdef HAVE_ATTROPEN
extern int attropenl();
#endif
extern int fstatatl();
#if defined(HAVE_LINKAT)
extern int linkatl();
#endif
extern int unlinkatl();
#if defined(HAVE_SYMLINKAT)
extern int symlinkatl();
#endif
#if defined(HAVE_READLINKAT)
extern int readlinkatl();
#endif
extern int fchownatl();
#if defined(HAVE_MKNODAT)
extern int mknodatl();
#endif
#if defined(HAVE_MKFIFOAT)
extern int mkfifoatl();
#endif
extern int rmdiratl();
#if defined(HAVE_MKDIRAT)
extern int mkdiratl();
#endif
extern int renameatl();
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
extern int futimesatl();
extern int utimensatl();
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
extern int lstatl();
extern int symlinkl();
extern int readlinkl();
extern int lchownl();
#endif

/* mkstemp */
extern char *mktempl();
extern char *mkdtempl();
extern int mkstempl();
extern int mkstempsl();
/* mkstemp GNU */
#if defined(_GNU_SOURCE)
extern int mkostempl();
extern int mkostempsl();
#endif

DIR *opendirl();

#ifdef HAVE_FGETXATTR
extern ssize_t getxattrl();
extern ssize_t lgetxattrl();
extern int setxattrl();
extern int lsetxattrl();
#endif

/* 64 */
#if     defined(_LARGEFILE64_SOURCE) && !((_FILE_OFFSET_BITS == 64) && \
            !defined(__PRAGMA_REDEFINE_EXTNAME))
extern int openl64();
extern int creatl64();
extern int statl64();

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
extern int openatl64();
#ifdef HAVE_ATTROPEN
extern int attropenl64();
#endif
extern int fstatatl64();
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
extern int lstatl64();
#endif

/* 64 mkstemp*/
extern int mkstempl64();
extern int mkstempsl64();
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
extern int mkostempl64();
extern int mkostempsl64();
#endif

#endif /* 64 */

#endif	/* defined(__STDC__) */


#endif /* _LONGPATH_H */
